import nltk

# Some data to make a model out of
text = """Mary had a little lamb,
His fleece was white as snow,
And everywhere that Mary went,
The lamb was sure to go.

He followed her to school one day,
Which was against the rule,
It made the children laugh and play
To see a lamb at school.

And so the teacher turned it out,
But still it lingered near,
And waited patiently about,
Till Mary did appear.

"Why does the lamb love Mary so?"
The eager children cry.
"Why, Mary loves the lamb, you know."
The teacher did reply."""

tokens = nltk.word_tokenize(text)


def unigram(tokens):
    """
    Function to build a unigram model out of some tokens
    :param tokens: data tokenized by nltk.word_tokenize()
    :return: the model, a dictionary from words to probabilities
    """
    model = {}
    for tok in tokens:
        # Check if we've seen this token before
        if tok in model:  # If we have, increase it's frequency by 1
            model[tok] += 1
        else:  # If we haven't, assign a frequency of 1
            model[tok] = 1
    for word in model:
        model[word] = model[word] / float(len(tokens))  # Normalize probabilities so they sum up to 1
    return model


def compute_perplexity(data, model):
    data = data.split()
    perplexity = 1  # Initialize value: starting is P = 1
    N = 0
    for word in data:
        N += 1
        if word not in model:
            model[word] = 0.00001  # Rudimentary smoothing
        perplexity = perplexity * (1 / model[word])
    perplexity = perplexity ** (1 / float(N))  # ** means power
    return perplexity


text_lamb = "a little lamb had Mary"
text_the = "the the the the the"
text1 = "This is a story about a lamb with white fleece who went to school."
text2 = "On the other hand if we just talk about cheeseburgers etc. the model will be more perplexed"
text3 = "Mais si on n'a pas des mots anglais la perplexité serait maximale!"

model = unigram(tokens)
print("All vocab in: " + str(compute_perplexity(text_lamb, model)))
print("The the the text: " + str(compute_perplexity(text_the, model)))
print("Some vocab overlap: " + str(compute_perplexity(text1, model)))
print("Same language: " + str(compute_perplexity(text2, model)))
print("French: " + str(compute_perplexity(text3, model)))
